@@ -16,7 +16,7 @@ import caching.base
 # Create your models here.
 
 
-from linghelper import DTW,getSemanticRelatedness
+from linghelper import DTW,disambiguate_sense,get_semantic_predictability
 from praatinterface import PraatLoader
 
 if 'phonostats' in settings.INSTALLED_APPS:
@@ -36,6 +36,8 @@ from .helper import fetch_buckeye_resource,reorganize
 from .managers import BulkManager
 
 
+
+
 GOOD_WORDS = ['back', 'bad', 'badge', 'bag', 'ball', 'bar', 'bare', 'base', 'bash', 'bass', 'bat', 'bath', 'beach', 'bean', 'bear', 'beat',
                  'bed', 'beer', 'bell', 'berth', 'big', 'bike', 'bill', 'birth', 'bitch', 'bite', 'boat', 'bob', 'boil', 'bomb', 'book', 'boom', 'boon',
                  'boss', 'bought', 'bout', 'bowl', 'buck', 'bum', 'burn', 'bus', 'bush', 'cab', 'cad', 'cake', 'calf', 'call', 'came', 'cap',
@@ -355,17 +357,42 @@ class WordType(caching.base.CachingMixin,models.Model):
     FWND = models.FloatField(blank=True,null=True)
     CVSkel = models.CharField(max_length=100,blank=True,null=True)
     StressVowel = models.CharField(max_length=10,blank=True,null=True)
+    Extra = PickledObjectField(null=True)
 
     objects = caching.base.CachingManager()
 
+    def get_celex_info(self):
+        if self.Extra is None:
+            self.Extra = {}
+        info = get_lexical_info(self.Label)
+        try:
+            self.Extra['CelexCategory'] = info['Cat']
+        except KeyError:
+            self.Extra['CelexCategory'] = 'NA'
+        try:
+            self.Extra['CelexFrequency'] = info['Freq']
+        except KeyError:
+            self.Extra['CelexFrequency'] = 'NA'
+        try:
+            self.Extra['CelexNeighDen'],self.Extra['CelexFWND'] = info['ND'], info['FWND']
+        except KeyError:
+            self.Extra['CelexNeighDen'],self.Extra['CelexFWND'] = 'NA','NA'
+        try:
+            self.Extra['CelexSPhoneProb'],self.Extra['CelexBiPhoneProb'] = info['SP'],info['BP']
+        except KeyError:
+            self.Extra['CelexSPhoneProb'],self.Extra['CelexBiPhoneProb'] = 'NA','NA'
+        self.save()
+
     def __unicode__(self):
         return u'%s' % self.Label
 
     def isAcceptable(self):
-        qs = self.wordtoken_set.filter(Output__isnull=False)
-        if len(qs) > 0:
-            if 'CelexCategory' in qs[0].Output and qs[0].Output['CelexCategory'] not in ['N','V','ADV','A']:
-                return False
+        if self.Extra is None or 'CelexCategory' not in self.Extra:
+            self.get_celex_info()
+        if self.Extra['CelexCategory'] not in ['N','V','ADV','A']:
+            return False
+        if sum([x.Vowel for x in self.UR.all()]) < 2:
+            return False
         return True
 
     def getUR(self,stressed=False,blick_style=False):
@@ -503,6 +530,18 @@ class WordType(caching.base.CachingMixin,models.Model):
         nd,fwnd = self.getNDs(subset=subset,speaker=speaker,dialog=dialog)
         return fwnd
 
+    def IDF(self):
+        if self.Extra is None:
+            self.Extra = {}
+        if 'IDF' in self.Extra:
+            return self.Extra['IDF']
+        base = WordToken.objects.all().select_related('WordType','Category')
+        freq = float(base.filter(WordType=self).count())/float(base.filter(~Q(Category__CategoryType='Other'),~Q(Category__CategoryType='Pause'),~Q(Category__CategoryType='Disfluency')).count())
+        idf = -1*math.log(freq)
+        self.Extra['IDF'] = idf
+        self.save()
+        return idf
+
     def getPhonoProb(self,subset='all',speaker=None,dialog=None):
         if subset == 'all' and self.SPhonoProb is not None and self.BiPhonoProb is not None:
             return self.SPhonoProb,self.BiPhonoProb
@@ -608,7 +647,25 @@ class WordToken(caching.base.CachingMixin,models.Model):
             self.setStrFormants()
         return self.StrVowelF2
 
-    def getPrevSemPred(self,style='A',window='A'):
+    def get_sense(self,disambiguate=True):
+        if self.Output is None:
+            self.Output = {}
+        if 'Wordnet_sense' not in self.Output:
+            cat = self.get_category(source='celex',style='wordnet')
+            if cat is None:
+                self.Output['Wordnet_sense'] = None
+                self.save()
+                return None
+            if not disambiguate:
+                return '.'.join([str(self),cat,'1'])
+            prev = self.getPreviousContext(window=10)
+            foll = self.getFollowingContext(window=10)
+            context = [[str(x),x.WordType.IDF()] for x in prev] + [[str(x),x.WordType.IDF()] for x in foll]
+            self.Output['Wordnet_sense'] = disambiguate_sense(str(self),cat,context,to_string=True)
+            self.save()
+        return self.Output['Wordnet_sense']
+
+    def getPrevSemPred(self,style='A',window='A',disambiguate=True):
         if style == 'A' and window == 'A':
             if self.PrevSemPred is not None:
                 return self.PrevSemPred
@@ -616,34 +673,11 @@ class WordToken(caching.base.CachingMixin,models.Model):
             prev_context = self.getPreviousContext(window = int(window))
         else:
             prev_context = self.getPreviousContext()
-        #catted = categorize_words([str(x)
-        #                        for x in prev_context
-        #                            if str(x) != str(self)]
-        #                        + [str(self)])
-
-        #cat_word = catted.pop(-1)
-        #tagged_context = ['#'.join([x,'1']) for x in
-        #                    catted
-        #                    if x[-2] == '#' and x[-1] in ['n','v','a','r']]
-        prev_context = filter(lambda x: x.getCelexCat() in ['N','V','A','ADV'],prev_context)
-        tagged_context = map(lambda x: '#'.join([str(x),
-                                                x.getCelexCat().lower().replace('adv','r'),
-                                                '1']),prev_context)
-        cat_word = '#'.join([str(self),self.getCelexCat(),'1'])
-        if cat_word.endswith('na#1'):
-            cat = self.Category.Label
-            if 'N' in cat:
-                cat = 'n'
-            elif 'JJ' in cat:
-                cat = 'a'
-            elif 'VB' in cat:
-                cat = 'v'
-            elif 'RB' in cat:
-                cat = 'r'
-            else:
-                cat = 'NA'
-            cat_word  = '#'.join([cat_word.split("#")[0],cat,'1'])
-        sp = getSemanticRelatedness(cat_word,tagged_context,style=style)
+        prev_context = filter(lambda x: x is not None,[ x.get_sense(disambiguate=disambiguate) for x in prev_context])
+        sense = self.get_sense(disambiguate=disambiguate)
+        if sense is None:
+            return None
+        sp = get_semantic_predictability(sense,prev_context,style=style)
         if style == 'A' and window == 'A':
             self.PrevSemPred = sp
             self.save()
@@ -658,34 +692,11 @@ class WordToken(caching.base.CachingMixin,models.Model):
             foll_context = self.getFollowingContext(window = int(window))
         else:
             foll_context = self.getFollowingContext()
-        #catted = categorize_words([str(x)
-        #                        for x in foll_context
-        #                            if str(x) != str(self)]
-        #                        + [str(self)])
-
-        #cat_word = catted.pop(-1)
-        #tagged_context = ['#'.join([x,'1']) for x in
-        #                    catted
-        #                    if x[-2] == '#' and x[-1] in ['n','v','a','r']]
-        foll_context = filter(lambda x: x.getCelexCat() in ['N','V','A','ADV'],foll_context)
-        tagged_context = map(lambda x: '#'.join([str(x),
-                                                x.getCelexCat().lower().replace('adv','r'),
-                                                '1']),foll_context)
-        cat_word = '#'.join([str(self),self.getCelexCat(),'1'])
-        if cat_word.endswith('na#1'):
-            cat = self.Category.Label
-            if 'N' in cat:
-                cat = 'n'
-            elif 'JJ' in cat:
-                cat = 'a'
-            elif 'VB' in cat:
-                cat = 'v'
-            elif 'RB' in cat:
-                cat = 'r'
-            else:
-                cat = 'NA'
-            cat_word  = '#'.join([cat_word.split("#")[0],cat,'1'])
-        sp = getSemanticRelatedness(cat_word,tagged_context,style=style)
+        foll_context = filter(lambda x: x is not None,[ x.get_sense(disambiguate=disambiguate) for x in foll_context])
+        sense = self.get_sense(disambiguate=disambiguate)
+        if sense is None:
+            return None
+        sp = get_semantic_predictability(sense,foll_context,style=style)
         if style == 'A' and window == 'A':
             self.FollSemPred = sp
             self.save()
@@ -974,6 +985,32 @@ class WordToken(caching.base.CachingMixin,models.Model):
             return False
         return True
 
+    def get_category(self,source='celex',style=None):
+        if source =='celex':
+            cat = self.getCelexCat()
+        else:
+            cat = str(self.Category)
+        if style == 'wordnet':
+            if source == 'celex':
+                if cat == 'ADV':
+                    cat = 'R'
+
+                if cat not in ['N','A','R','V']:
+                    return None
+                cat = cat.lower()
+            else:
+                if 'N' in cat:
+                    cat = 'n'
+                elif 'JJ' in cat:
+                    cat = 'a'
+                elif 'VB' in cat:
+                    cat = 'v'
+                elif 'RB' in cat:
+                    cat = 'r'
+                else:
+                    return None
+        return cat
+
     def getCelexCat(self):
         if self.Output is None:
             self.Output = {}
